;
; *.*
;   OCM-EXTRA Pack / Switched I/O ports ($40-$4F) Revision 5 or later
;
; Copyright (c) 2017-2022 KdL
; All rights reserved.
;
; Redistribution and use of this source code or any derivative works, are
; permitted provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright notice,
;    this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
; 3. Redistributions may not be sold, nor may they be used in a commercial
;    product or activity without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
; TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; ----------------------------------------
;     Prog: OCM-FIRMINFO v1.1 by KdL
;     Date: 2022.09.24
;     Note: OCM-PLD Pack v3.4 or later
;     Coded in TWZ'CA3 w/ TASM80 v3.2ud
; ----------------------------------------
;

            .ORG  $0100

FN1         .EQU  'f'                     ; strings and options
FN2         .EQU  'i'
FN3         .EQU  'r'
FN4         .EQU  'm'
FN5         .EQU  'i'
FN6         .EQU  'n'
FN7         .EQU  'f'
FN8         .EQU  'o'
VER1        .EQU  '1'
VER2        .EQU  '1'
YR1         .EQU  '2'
YR2         .EQU  '0'
YR3         .EQU  '2'
YR4         .EQU  '2'
VERBOSE1    .EQU  '/'
VERBOSE2    .EQU  '-'
OPT1        .EQU  '1'
OPTR        .EQU  'r'
SYSTYPE     .EQU  't'
HLP1        .EQU  'h'
HLP2        .EQU  '?'

SCMDOPT1    .EQU  $00                     ; null
SCMDBASE    .EQU  OPT1-SCMDOPT1
SWIOADDR    .EQU  $40                     ; device id register of switched i/o ports
MACTYPE     .EQU  $49
VERADDR     .EQU  $4e
REVADDR     .EQU  $4f
NULID       .EQU  $80
OCMID       .EQU  $d4                     ; ocm id

LOWERCASE   .EQU  %00100000               ; masks
UP          .EQU  -LOWERCASE
_           .EQU  %10000000               ; obfuscation mask, e.g. 'A'|_,'B'|_
DEOBF       .EQU  %01111111               ; de-obfuscation mask
MACMASK     .EQU  %00111100
VERMASK     .EQU  %01100000
REVMASK     .EQU  %00011111
CHARZERO    .EQU  '0'
CHARDOT     .EQU  '.'
TWO         .EQU  $02
THREE       .EQU  $03
FOUR        .EQU  $04
FIVE        .EQU  $05
SIX         .EQU  $06
TEN         .EQU  $0a

NUL         .EQU  $00                     ; ascii table
LF          .EQU  $0a
CLS         .EQU  $0c
CR          .EQU  $0d
EOF         .EQU  $1a
SPC         .EQU  $20

PLENGHT     .EQU  $0080                   ; system calls
CALSLT      .EQU  $001c
CHPUT       .EQU  $00a2
EXPTBL      .EQU  $fcc1
_BDOS       .EQU  $0005
_STROUT     .EQU  $09                     ; string output

ONE         .EQU  $01                     ;   1 byte  for char
HLN         .EQU  $10                     ;  16 bytes for hex line
CLU         .EQU  $0100                   ; 256 bytes for cluster

K1          .EQU  'K'                     ; editable copyleft: 1st three chars
K2          .EQU  'd'
K3          .EQU  'L'
_1          .EQU  K1|_                    ; obfuscated copyleft
_2          .EQU  K2|_
_3          .EQU  K3|_
; ----------------------------------------
startProg:
            jr    mainProg                ; type support, A:\>TYPE FILENAME.COM
typeMsg:
;           .DB   CR,SPC,SPC,SPC,SPC,LF   ; 'jr mainProg' ctrl sequences mask
            .DB   CR,"OCM-"
            .DB   FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   " v",VER1,".",VER2," by "
clearCopyleft:
            .DB   K1,K2,K3," 2017-",YR1,YR2,YR3,YR4,CR
msgLF:
            .DB   LF,EOF
; ----------------------------------------
EXTRA:      .DB   NUL                     ; vars
; ----------------------------------------
mainProg:
            di
            ld    a,OCMID
            out   (SWIOADDR),a            ; set OCM ID
            ld    c,LF+ONE                ; init 'call strDisp' to use
            ld    d,NULID+SCMDBASE        ; reg d is 'OPTION'
            ld    e,VERBOSE1              ; reg e is 'PREFIX'
copyleftChk:
            ld    hl,clearCopyleft
            ld    a,_1
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
            jr    nz,copyleftNext         ; protection: 'K' is confirmed here
            inc   hl
            ld    a,_2
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
copyleftNext:
            jr    nz,jrnzSPC              ; protection: 'd' is confirmed here
            inc   hl
            ld    a,_3
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
jrnzSPC:
            jr    nz,noParam              ; protection: parameter down!
; ----------------------------------------
startParser:
            ld    hl,PLENGHT
            ld    b,(hl)
            ld    a,b
            cp    NUL
            jr    z,noParam               ; jump if no chars
            ld    a,(PLENGHT+ONE)
            cp    SPC                     ; ignore this policy:
            jr    nz,noParam              ; e.g. A:\>FILENAME/x
preLoop:
            inc   hl
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,getParam             ; get 1st parameter:
            djnz  preLoop                 ; e.g. A:\>FILENAME      -x
noParam:
            jp    ocmidChk                ; no parameters
getParam:
            ld    a,(hl)
            ld    e,a
            djnz  getOption               ; if no option, get invalid parameter
invalidParam:
            ld    hl,errorMsg0
            call  strDisp                 ; 'invalid parameter' without prefix
            jr    nearestExit             ; jump to the nearest exit, 2 bytes
getOption:
            inc   hl
            ld    a,(hl)
            or    LOWERCASE               ; 1st option forced to lowercase
            ld    d,a
            djnz  nextOption              ; if end of PLENGHT, parameter is get
            jr    prefixChk
nextOption:
            inc   hl
            ld    a,(hl)
            or    LOWERCASE               ; 2nd option forced to lowercase
            cp    SYSTYPE
            jr    nz,postLoop1            ; is other
set2ndOption:
            ld    (EXTRA),a               ; is 't'
            jr    postLoop2
postLoop:
            inc   hl
postLoop1:
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,invalidParam         ; get 2nd parameter as invalid:
postLoop2:
            djnz  postLoop                ; e.g. A:\>FILENAME -x    abcdef
prefixChk:
            ld    a,e
            cp    VERBOSE2                ; '-'
            jr    z,paramChk
            cp    VERBOSE1                ; '/'
            jr    nz,invalidParam
; ----------------------------------------
paramChk:
            ld    a,d
            cp    OPTR
            jr    z,ocmidChk1             ; jump if 'r'
            ld    a,(EXTRA)
            cp    b
            jr    nz,invalidParam         ; invalid with '/tr', '-t?', etc.
            ld    a,d
            cp    SYSTYPE
            jr    z,extraOnly             ; jump if 't'
            cp    HLP1
            jr    z,isHelp                ; jump if 'h'
            cp    HLP2
            jr    nz,invalidOption        ; jump if '?'
isHelp:
            ld    hl,helpMsg              ; help message
nearestExit:
            jp    lastDisp
invalidOption:
            ld    hl,errorMsg0
            call  strDisp
            ld    hl,errorMsg2
            jr    nearestExit             ; 'invalid option'
extraOnly:
            ld    (EXTRA),a               ; shift the option to extra
            ld    d,NULID+SCMDBASE        ; $80 is the null smart command
            jr    ocmidChk
; ----------------------------------------
ocmidChk1:                                ; set I/O revision number
            ld    d,OPT1
            in    a,(REVADDR)
            and   REVMASK
            ld    hl,revisionNum
            call  byteToDecimal
            ld    a,DEOBF
            ld    (revisionNum),a         ; = 2 * backspace
            ld    a,(revisionNum+1)
            cp    CHARZERO
            jr    nz,ocmidChk
            ld    a,(revisionNum-1)
            inc   a
            ld    (revisionNum-1),a       ; = 1 * space
            ld    a,DEOBF
            ld    (revisionNum+1),a       ; = 2 * backspace
; ----------------------------------------
ocmidChk:
            in    a,(SWIOADDR)            ; check for OCM ID
            cpl
            cp    OCMID
; ----------------------------------------
            ld    hl,genericMsg           ; hl <= 'generic firmware'
            jr    nz,nearestExit          ; 'generic firmware' (rem to test)
            in    a,(REVADDR)             ; revision check
            and   REVMASK
            cp    NUL                     ; = revision 0
            jr    z,nearestExit           ; 'generic firmware' (jr nz to test)
; ----------------------------------------
            ld    hl,earlierMsg           ; hl <= 'earlier revision'
            ld    b,4                     ; = revision 4, 3, 2, 1
testEarlier:
            cp    b
            jr    z,printEarlier          ; 'earlier revision' (jr nz to test)
            djnz  testEarlier
; ----------------------------------------
            in    a,(VERADDR)
            ld    hl,versionNum
            call  byteToDecimal           ; set version XX.Y numbers

            ld    a,(versionNum)          ; 1st digit = 0 ???
            cp    CHARZERO
            jr    nz,setVersZ
            ld    a,(versionNum-1)
            inc   a
            ld    (versionNum-1),a        ; = 1 * space
            ld    a,DEOBF
            ld    (versionNum),a          ; = 2 * backspace
setVersZ:
            ld    a,(versionNum+TWO)
            ld    (versionNum+THREE),a
            ld    a,CHARDOT
            ld    (versionNum+TWO),a      ; 'version XX.Y'
            in    a,(REVADDR)
            and   VERMASK
            cp    NUL
            jr    nz,printVersZ           ; set (or skip) version .Z number
            ld    a,DEOBF
            ld    (versionNum+SIX),a      ; = 2 * backspace
            jr    printVersion
printVersZ:
            rrca
            rrca
            rrca
            rrca
            rrca
            or    CHARZERO
            ld    (versionNum+FIVE),a
            ld    a,CHARDOT
            ld    (versionNum+FOUR),a     ; 'version XX.Y.Z'
printVersion:
            ld    hl,versionMsg           ; 'OCM-PLD version XX.Y.Z by KdL'
printEarlier:
            ld    a,d
            cp    OPT1
            jr    nz,printSysType
printRevision:
            ld    a,e
            cp    VERBOSE2
            jr    z,singleRow
            call  strDisp
            ld    hl,revisionMsg0         ; 'Switched I/O ports Revision NN'
            jr    printSysType
singleRow:
            call  strDisp
            ld    hl,parenthesisIn        ; ' (Switched I/O ports Revision NN)'
            call  strDisp
            ld    hl,revisionMsg1
            call  strDisp
            ld    hl,parenthesisOut
printSysType:
            ld    a,(EXTRA)
            cp    SYSTYPE
            jr    nz,lastDisp
            call  strDisp
            ld    hl,systemMsg            ; 'System type: MSX++ '
            call  strDisp
            in    a,(MACTYPE)
            and   MACMASK
            cp    NUL
            jr    nz,printType2
            ld    hl,typeMsg1             ; 'computer'
            jr    lastDisp
printType2:
            ld    hl,typeMsg2             ; 'compatible'

; ----------------------------------------
lastDisp:
            dec   c                       ; use 'jr lastDisp' or 'jp lastDisp'
strDisp:
            inc   c                       ; use 'call strDisp'
charLoop:
            ld    a,(hl)                  ; load 1st char, ascii < 127 only
            and   DEOBF                   ; de-obfuscation
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; NUL and EOF are not valid as 1st
            inc   hl
            xor   a
            cp    (hl)
            jr    nz,nextChar             ; NUL,n = SPC(n)
            inc   hl
            ld    b,(hl)                  ; b = n
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
;           and   DEOBF                   ; de-obfuscation (optional)
spcLoop:
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; print SPC until b = 0
            djnz  spcLoop
            inc   hl
nextChar:
            ld    a,EOF
            cp    (hl)
            jr    nz,charLoop             ; EOF = end of string
            inc   hl                      ; point to the next string at exit
            dec   c
            ld    a,LF
            cp    c                       ; c = LF is 'lastDisp'
            ret   nz                      ; exit of 'strDisp'
            ld    de,strLFCR              ; print LF + CR after 'lastDisp'
            ld    c,_STROUT               ; 'OPTION' and 'PREFIX' will be lost
            call  _BDOS
mainExit:
            xor   a
            out   (SWIOADDR),a            ; cancel Device ID
            ld    (PLENGHT),a             ; reset PLENGHT for the next command
            ei
            ret
strLFCR:
            .DB   LF,CR,'$'
; ----------------------------------------
byteToDecimal:
            push  bc
            push  de
            ex    de,hl
            ld    l,a
            ld    h,NUL
            ld    bc,-TEN*TEN             ; bc = -100
            call  setDigit                ; set 1st digit
            ld    c,-TEN                  ; bc = -10
            call  setDigit                ; set 2nd digit
            ld    c,b
            call  setDigit                ; set 3rd digit
            pop   de
            pop   bc
            ret
setDigit:
            ld    a,CHARZERO-ONE
testNumber:
            inc   a
            add   hl,bc
            jr    c,testNumber
            sbc   hl,bc
            ld    (de),a
            inc   de
            ret
; ----------------------------------------
helpMsg:                                  ; texts comply with 37 columns
            .DB   LF|_
            .DB   CLS|_,'S'|_,'y'|_,'n'|_,'t'|_,'a'|_,'x'|_,':'|_,' '|_
            .DB   FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   '['|_,'{'|_,VERBOSE1|_,'|'|_,VERBOSE2|_,'}'|_
            .DB   'o'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,']'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'N'|_,'o'|_,' '|_
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,':'|_
            .DB   NUL,4,'S'|_,'h'|_,'o'|_,'w'|_,'s'|_,' '|_,'t'|_,'h'|_,'e'|_
            .DB   ' '|_,'c'|_,'u'|_,'r'|_,'r'|_,'e'|_,'n'|_,'t'|_
            .DB   LF|_,CR|_,NUL,17,'f'|_,'i'|_,'r'|_,'m'|_,'w'|_,'a'|_,'r'|_
            .DB   'e'|_,' '|_,'v'|_,'e'|_,'r'|_,'s'|_,'i'|_,'o'|_,'n'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'P'|_,'r'|_,'e'|_,'f'|_,'i'|_,'x'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,4,VERBOSE1|_,NUL,3
            .DB   'V'|_,'e'|_,'r'|_,'b'|_,'o'|_,'s'|_,'e'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_,' '|_,'#'|_,'1'|_
            .DB   LF|_,CR|_,NUL,13,VERBOSE2|_,NUL,3
            .DB   'V'|_,'e'|_,'r'|_,'b'|_,'o'|_,'s'|_,'e'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_,' '|_,'#'|_,'2'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'O'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,'s'|_,':'|_
            .DB   NUL,5,OPTR|_,NUL,3,'A'|_,'d'|_,'d'|_,'s'|_,' '|_,'I'|_,'/'|_
            .DB   'O'|_,' '|_,'r'|_,'e'|_,'v'|_,'i'|_,'s'|_,'i'|_,'o'|_,'n'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'E'|_,'x'|_,'t'|_,'r'|_,'a'|_,'s'|_,':'|_,NUL,6
            .DB   SYSTYPE|_,NUL,3,'M'|_,'S'|_,'X'|_,'+'|_,'+'|_,' '|_,'s'|_
            .DB   'y'|_,'s'|_,'t'|_,'e'|_,'m'|_,' '|_,'t'|_,'y'|_,'p'|_,'e'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'E'|_,'x'|_,'a'|_,'m'|_,'p'|_,'l'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,8,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   LF|_,CR|_
            .DB   NUL,17,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   VERBOSE1|_,OPTR|_,LF|_,CR|_
            .DB   NUL,17,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   VERBOSE2|_,OPTR|_,SYSTYPE|_

            .DB   EOF
versionMsg:
            .DB   'O'|_,'C'|_,'M'|_,'-'|_,'P'|_,'L'|_,'D'|_,' '|_
            .DB   'v'|_,'e'|_,'r'|_,'s'|_,'i'|_,'o'|_,'n'|_,NUL,1
versionNum:
            .DB   '?'|_,'?'|_,'.'|_,'?'|_,' '|_,' '|_
            .DB   ' '|_,'b'|_,'y'|_,' '|_,_1,_2,_3,EOF
revisionMsg0:
            .DB   LF|_,CR|_
revisionMsg1:
            .DB   'S'|_,'w'|_,'i'|_,'t'|_,'c'|_,'h'|_,'e'|_,'d'|_,' '|_
            .DB   'I'|_,'/'|_,'O'|_,' '|_,'p'|_,'o'|_,'r'|_,'t'|_,'s'|_,' '|_
            .DB   'R'|_,'e'|_,'v'|_,'i'|_,'s'|_,'i'|_,'o'|_,'n'|_,NUL,2
revisionNum:
            .DB   DEOBF,'?'|_,'?'|_,EOF
systemMsg:
            .DB   LF|_,CR|_,LF|_
            .DB   'S'|_,'y'|_,'s'|_,'t'|_,'e'|_,'m'|_,' '|_
            .DB   't'|_,'y'|_,'p'|_,'e'|_,':'|_,' '|_
            .DB   'M'|_,'S'|_,'X'|_,'+'|_,'+'|_,' '|_,EOF
typeMsg1:
            .DB   'c'|_,'o'|_,'m'|_,'p'|_,'u'|_,'t'|_,'e'|_,'r'|_,EOF
typeMsg2:
            .DB   'c'|_,'o'|_,'m'|_,'p'|_,'a'|_,'t'|_,'i'|_,'b'|_,'l'|_,'e'|_,EOF
parenthesisIn:
            .DB   ' '|_,'('|_,EOF
parenthesisOut:
            .DB   ')'|_,EOF
earlierMsg:
            .DB   'O'|_,'C'|_,'M'|_,'-'|_,'P'|_,'L'|_,'D'|_,' '|_
            .DB   'v'|_,'3'|_,'.'|_,'3'|_,'.'|_,'3'|_,' '|_,'o'|_,'r'|_,' '|_
            .DB   'e'|_,'a'|_,'r'|_,'l'|_,'i'|_,'e'|_,'r'|_
            .DB   ' '|_,'b'|_,'y'|_,' '|_,_1,_2,_3,EOF
errorMsg0:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'I'|_,'n'|_,'v'|_,'a'|_,'l'|_,'i'|_,'d'|_,' '|_,EOF
errorMsg1:
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,EOF
errorMsg2:
            .DB   'o'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,EOF
genericMsg:
            .DB   'U'|_,'n'|_,'k'|_,'n'|_,'o'|_,'w'|_,'n'|_,' '|_,'f'|_,'i'|_
            .DB   'r'|_,'m'|_,'w'|_,'a'|_,'r'|_,'e'|_,' '|_,'o'|_,'r'|_,' '|_
            .DB   'o'|_,'t'|_,'h'|_,'e'|_,'r'|_,' '|_,'M'|_,'S'|_,'X'|_,' '|_
            .DB   's'|_,'y'|_,'s'|_,'t'|_,'e'|_,'m'|_,EOF
; ----------------------------------------
startFill:                                ; $FF fill the 1st hex line
            .FILL ((((startFill-startProg)/HLN)+ONE)*HLN-(startFill-startProg))
endProg:                                  ; $FF fill the cluster
;           .FILL ((((endProg-startProg)/CLU)+ONE)*CLU-(endProg-startProg))-HLN
                                          ; [ FILENAME.COM ] at last hex line
            .DB   "[ ",FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   ".COM ]"

            .END

