;
; *.*
;   OCM-EXTRA Pack / Switched I/O ports ($40-$4F) Revision 3 or later
;
; Copyright (c) 2011-2022 KdL
; All rights reserved.
;
; Redistribution and use of this source code or any derivative works, are
; permitted provided that the following conditions are met:
;
; 1. Redistributions of source code must retain the above copyright notice,
;    this list of conditions and the following disclaimer.
; 2. Redistributions in binary form must reproduce the above copyright
;    notice, this list of conditions and the following disclaimer in the
;    documentation and/or other materials provided with the distribution.
; 3. Redistributions may not be sold, nor may they be used in a commercial
;    product or activity without specific prior written permission.
;
; THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
; "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
; TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
; PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
; CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
; EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
; PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
; WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
; OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
; ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
;
; ----------------------------------------
;     Prog: OCM-SETAUDIO v1.5 by KdL
;     Date: 2022.09.24
;     Note: OCM-PLD Pack v3.1 or later
;     Coded in TWZ'CA3 w/ TASM80 v3.2ud
; ----------------------------------------
;

            .ORG  $0100

FN1         .EQU  's'                     ; strings and options
FN2         .EQU  'e'
FN3         .EQU  't'
FN4         .EQU  'a'
FN5         .EQU  'u'
FN6         .EQU  'd'
FN7         .EQU  'i'
FN8         .EQU  'o'
VER1        .EQU  '1'
VER2        .EQU  '5'
YR1         .EQU  '2'
YR2         .EQU  '0'
YR3         .EQU  '2'
YR4         .EQU  '2'
VERBOSE     .EQU  '/'
QUIET       .EQU  '-'
OPT1        .EQU  '1'
OPT2        .EQU  '2'
OPTM        .EQU  'm'
OPTS        .EQU  's'
HLP1        .EQU  'h'
HLP2        .EQU  '?'

SCMDOPT1    .EQU  $3d
SCMDBASE    .EQU  OPT1-SCMDOPT1
SWIOADDR    .EQU  $40                     ; device id register of switched i/o ports
SCMDADDR    .EQU  $41                     ; smart command port
STATADDR    .EQU  $49
REVADDR     .EQU  $4f
NULID       .EQU  $80
OCMID       .EQU  $d4                     ; ocm id

STERMODE    .EQU  %00000001               ; masks
LOWERCASE   .EQU  %00100000
UP          .EQU  -LOWERCASE
_           .EQU  %10000000               ; obfuscation mask, e.g. 'A'|_,'B'|_
DEOBF       .EQU  %01111111               ; de-obfuscation mask
REVMASK     .EQU  %00011111

NUL         .EQU  $00                     ; ascii table
LF          .EQU  $0a
CLS         .EQU  $0c
CR          .EQU  $0d
EOF         .EQU  $1a
SPC         .EQU  $20

PLENGHT     .EQU  $0080                   ; system calls
CALSLT      .EQU  $001c
CHPUT       .EQU  $00a2
EXPTBL      .EQU  $fcc1
_BDOS       .EQU  $0005
_STROUT     .EQU  $09                     ; string output

ONE         .EQU  $01                     ;   1 byte  for char
HLN         .EQU  $10                     ;  16 bytes for hex line
CLU         .EQU  $0100                   ; 256 bytes for cluster

K1          .EQU  'K'                     ; editable copyleft: 1st three chars
K2          .EQU  'd'
K3          .EQU  'L'
_1          .EQU  K1|_                    ; obfuscated copyleft
_2          .EQU  K2|_
_3          .EQU  K3|_
; ----------------------------------------
startProg:
            jr    mainProg                ; type support, A:\>TYPE FILENAME.COM
typeMsg:
;           .DB   CR,SPC,SPC,SPC,SPC,LF   ; 'jr mainProg' ctrl sequences mask
            .DB   CR,"OCM-"
            .DB   FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   " v",VER1,".",VER2," by "
clearCopyleft:
            .DB   K1,K2,K3," 2011-",YR1,YR2,YR3,YR4,CR
msgLF:
            .DB   LF,EOF
; ----------------------------------------
mainProg:
            di
            ld    a,OCMID
            out   (SWIOADDR),a            ; set OCM ID
            ld    c,LF+ONE                ; init 'call strDisp' to use
            ld    d,NULID+SCMDBASE        ; reg d is 'OPTION'
            ld    e,VERBOSE               ; reg e is 'PREFIX'
copyleftChk:
            ld    hl,clearCopyleft
            ld    a,_1
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
            jr    nz,copyleftNext         ; protection: 'K' is confirmed here
            inc   hl
            ld    a,_2
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
copyleftNext:
            jr    nz,jrnzSPC              ; protection: 'd' is confirmed here
            inc   hl
            ld    a,_3
            and   DEOBF                   ; de-obfuscation
            cp    (hl)
jrnzSPC:
            jr    nz,noParam              ; protection: parameter down!
; ----------------------------------------
startParser:
            ld    hl,PLENGHT
            ld    b,(hl)
            ld    a,b
            cp    NUL
            jr    z,noParam               ; jump if no chars
            ld    a,(PLENGHT+ONE)
            cp    SPC                     ; ignore this policy:
            jr    nz,noParam              ; e.g. A:\>FILENAME/x
preLoop:
            inc   hl
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,getParam             ; get 1st parameter:
            djnz  preLoop                 ; e.g. A:\>FILENAME      -x
noParam:
            jr    ocmidChk                ; no parameters
getParam:
            ld    a,(hl)
            ld    e,a
            djnz  getOption               ; if no option, get invalid parameter
invalidParam:
            ld    hl,errorMsg0
            call  strDisp                 ; 'invalid parameter' without prefix
            jr    nearestExit             ; jump to the nearest exit, 2 bytes
getOption:
            inc   hl
            ld    a,(hl)
            or    LOWERCASE               ; 1st option forced to lowercase
            ld    d,a
            djnz  postLoop                ; if end of PLENGHT, parameter is get
            jr    prefixChk
postLoop:
            inc   hl
postLoop1:
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
            cp    (hl)
            jr    nz,invalidParam         ; get 2nd parameter as invalid:
postLoop2:
            djnz  postLoop                ; e.g. A:\>FILENAME -x    abcdef
prefixChk:
            ld    a,e
            cp    QUIET                   ; '-'
            jr    z,paramChk
            cp    VERBOSE                 ; '/'
            jr    nz,invalidParam
; ----------------------------------------
paramChk:
            ld    a,d
            cp    OPTM
            jr    z,ocmidChk1             ; jump if 'm'
            cp    OPTS
            jr    z,ocmidChk2             ; jump if 's'
            cp    HLP1
            jr    z,isHelp                ; jump if 'h'
            cp    HLP2
            jr    nz,invalidOption        ; jump if '?'
isHelp:
            ld    hl,helpMsg              ; help message
nearestExit:
            jr    lastDisp
invalidOption:
            ld    hl,errorMsg0
            call  strDisp
            ld    hl,errorMsg2
            jr    lastDisp                ; 'invalid option'
ocmidChk1:
            ld    d,OPT1
            jr    ocmidChk
ocmidChk2:
            ld    d,OPT2
; ----------------------------------------
ocmidChk:
            in    a,(SWIOADDR)            ; check for OCM ID
            cpl
            cp    OCMID
; ----------------------------------------
            jr    nz,isUnsupp             ; 'unsupported command' (rem to test)
            in    a,(REVADDR)             ; revision check
            and   REVMASK
            cp    NUL                     ; = revision 0
            jr    z,isUnsupp              ; 'unsupported command' (jr nz to test)
; ----------------------------------------
            ld    b,2                     ; = revision 2, 1
testUnsupp:
            cp    b
            jr    z,isUnsupp              ; 'unsupported command' (jr nz to test)
            djnz  testUnsupp
; ----------------------------------------
            ld    a,d
outScmd:
            sub   SCMDBASE
            out   (SCMDADDR),a            ; set smart command
quietChk:
            ld    a,e
            cp    QUIET
            jr    z,quietExit             ; quiet exit
; ----------------------------------------
getStatus:
            in    a,(STATADDR)
            and   STERMODE
            cp    NUL
            jr    z,isMono                ; STATADDR is 'xxxxxxx0'
isStereo:
            ld    hl,msgStereo            ; STATADDR is 'xxxxxxx1'
            jr    lastDisp                ; 'Stereo mode'
isMono:
            ld    hl,msgMono              ; 'Mono'
            call  strDisp
            ld    hl,msgMode              ; ' mode'
            jr    lastDisp                ; display last string and exit
isUnsupp:
            ld    a,e
            cp    QUIET
quietExit:
            jr    z,mainExit              ; quiet exit (useful for batch)
            ld    hl,unsuppMsg            ; 'unsupported command' if '/' prefix
; ----------------------------------------
lastDisp:
            dec   c                       ; use 'jr lastDisp' or 'jp lastDisp'
strDisp:
            inc   c                       ; use 'call strDisp'
charLoop:
            ld    a,(hl)                  ; load 1st char, ascii < 127 only
            and   DEOBF                   ; de-obfuscation
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; NUL and EOF are not valid as 1st
            inc   hl
            xor   a
            cp    (hl)
            jr    nz,nextChar             ; NUL,n = SPC(n)
            inc   hl
            ld    b,(hl)                  ; b = n
            ld    a,(jrnzSPC)             ; protection: jrnzSPC = $20
;           and   DEOBF                   ; de-obfuscation (optional)
spcLoop:
            ld    ix,CHPUT                ; init chput
            ld    iy,(EXPTBL-1)           ; init main-rom slot
            call  CALSLT                  ; print SPC until b = 0
            djnz  spcLoop
            inc   hl
nextChar:
            ld    a,EOF
            cp    (hl)
            jr    nz,charLoop             ; EOF = end of string
            inc   hl                      ; point to the next string at exit
            dec   c
            ld    a,LF
            cp    c                       ; c = LF is 'lastDisp'
            ret   nz                      ; exit of 'strDisp'
            ld    de,strLFCR              ; print LF + CR after 'lastDisp'
            ld    c,_STROUT               ; 'OPTION' and 'PREFIX' will be lost
            call  _BDOS
mainExit:
            xor   a
            out   (SWIOADDR),a            ; cancel Device ID
            ld    (PLENGHT),a             ; reset PLENGHT for the next command
            ei
            ret
strLFCR:
            .DB   LF,CR,'$'
; ----------------------------------------
helpMsg:                                  ; texts comply with 37 columns
            .DB   LF|_
            .DB   CLS|_,'S'|_,'y'|_,'n'|_,'t'|_,'a'|_,'x'|_,':'|_,' '|_
            .DB   FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   '['|_,'{'|_,VERBOSE|_,'|'|_,QUIET|_,'}'|_
            .DB   'o'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,']'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'N'|_,'o'|_,' '|_
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,':'|_
            .DB   NUL,4,'S'|_,'h'|_,'o'|_,'w'|_,'s'|_,' '|_,'t'|_,'h'|_,'e'|_
            .DB   ' '|_,'s'|_,'t'|_,'a'|_,'t'|_,'u'|_,'s'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'P'|_,'r'|_,'e'|_,'f'|_,'i'|_,'x'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,4,VERBOSE|_,NUL,3
            .DB   'V'|_,'e'|_,'r'|_,'b'|_,'o'|_,'s'|_,'e'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_
            .DB   LF|_,CR|_,NUL,13,QUIET|_,NUL,3
            .DB   'Q'|_,'u'|_,'i'|_,'e'|_,'t'|_,' '|_
            .DB   'o'|_,'u'|_,'t'|_,'p'|_,'u'|_,'t'|_
            .DB   LF|_

            .DB   LF|_,CR|_,'O'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,'s'|_,':'|_
            .DB   NUL,5,OPTM|_,NUL,3,'M'|_,'o'|_,'n'|_,'o'|_,' '|_
            .DB   'm'|_,'o'|_,'d'|_,'e'|_,LF|_,CR|_,NUL,13,OPTS|_,NUL,3
            .DB   'S'|_,'t'|_,'e'|_,'r'|_,'e'|_,'o'|_
            .DB   ' '|_,'m'|_,'o'|_,'d'|_,'e'|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'E'|_,'x'|_,'a'|_,'m'|_,'p'|_,'l'|_,'e'|_,'s'|_,':'|_
            .DB   NUL,8,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   VERBOSE|_,OPTM|_,LF|_,CR|_
            .DB   NUL,17,FN1|_,FN2|_,FN3|_,FN4|_,FN5|_,FN6|_,FN7|_,FN8|_,' '|_
            .DB   QUIET|_,OPTS|_
            .DB   LF|_

            .DB   LF|_,CR|_
            .DB   'P'|_,'l'|_,'u'|_,'g'|_,'s'|_,':'|_,' '|_
            .DB   'W'|_,'h'|_,'i'|_,'t'|_,'e'|_,'/'|_,'L'|_,'C'|_,'H'|_,' '|_
            .DB   'A'|_,'u'|_,'d'|_,'i'|_,'o'|_,' '|_
            .DB   'c'|_,'a'|_,'r'|_,'t'|_,'r'|_,'i'|_,'d'|_,'g'|_,'e'|_,'s'|_
            .DB   LF|_,CR|_,NUL,9,'R'|_,'e'|_,'d'|_,'/'|_,'R'|_,'C'|_,'H'|_
            .DB   ' '|_,'I'|_,'n'|_,'t'|_,'e'|_,'r'|_,'n'|_,'a'|_,'l'|_
            .DB   ' '|_,'s'|_,'o'|_,'u'|_,'n'|_,'d'|_,'s'|_

            .DB   EOF
msgMono:
            .DB   'M'|_,'o'|_,'n'|_,'o'|_,EOF
msgStereo:
            .DB   'S'|_,'t'|_,'e'|_,'r'|_,'e'|_,'o'|_
msgMode:
            .DB   ' '|_,'m'|_,'o'|_,'d'|_,'e'|_,EOF
errorMsg0:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'I'|_,'n'|_,'v'|_,'a'|_,'l'|_,'i'|_,'d'|_,' '|_,EOF
errorMsg1:
            .DB   'p'|_,'a'|_,'r'|_,'a'|_,'m'|_,'e'|_,'t'|_,'e'|_,'r'|_,EOF
errorMsg2:
            .DB   'o'|_,'p'|_,'t'|_,'i'|_,'o'|_,'n'|_,EOF
unsuppMsg:
            .DB   LF|_,'*'|_,'*'|_,'*'|_,' '|_
            .DB   'U'|_,'n'|_,'s'|_,'u'|_,'p'|_,'p'|_,'o'|_,'r'|_,'t'|_,'e'|_
            .DB   'd'|_,' '|_,'c'|_,'o'|_,'m'|_,'m'|_,'a'|_,'n'|_,'d'|_,EOF
; ----------------------------------------
startFill:                                ; $FF fill the 1st hex line
            .FILL ((((startFill-startProg)/HLN)+ONE)*HLN-(startFill-startProg))
endProg:                                  ; $FF fill the cluster
;           .FILL ((((endProg-startProg)/CLU)+ONE)*CLU-(endProg-startProg))-HLN
                                          ; [ FILENAME.COM ] at last hex line
            .DB   "[ ",FN1+UP,FN2+UP,FN3+UP,FN4+UP,FN5+UP,FN6+UP,FN7+UP,FN8+UP
            .DB   ".COM ]"

            .END
